#version 430 core
layout (location = 0) out vec4 FragColor;
layout (location = 1) out vec4 BrightColor;



in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

struct Light {
    vec3 Position;
    vec3 Color;
};


uniform Light light;

uniform sampler2D diffuseTexture;
uniform vec3 viewPos;

void main()
{           
    vec3 color = texture(diffuseTexture, vert.uv).rgb;
    vec3 normal = normalize(vert.normal_world);

    // ambient
    vec3 ambient = 0.0 * color;

    // lighting
    vec3 lighting = vec3(0.0);
    vec3 viewDir = normalize(viewPos - vert.position_world);
   
		//LIGHT 	
        // diffuse
        vec3 lightDir = normalize(light.Position - vert.position_world);
        float diff = max(dot(lightDir, normal), 0.0);
        vec3 result = light.Color * diff * color;   
		
        // attenuation (use quadratic as we have gamma correction)
        float distance = length(vert.position_world - light.Position);
        result *= 1.0 / (distance * distance);
		 lighting += result;
                
    
     vec3 finalResult = ambient + lighting;

    // check whether finalResult is higher than some threshold, if so, output as bloom threshold color
    float brightness = dot(finalResult, vec3(0.2126, 0.7152, 0.0722));
    if(brightness > 0.5)
       BrightColor = vec4(finalResult, 1.0);
    else
        BrightColor = vec4(0.0, 0.0, 0.0, 1.0);

    FragColor = vec4(finalResult, 1.0);

}