#version 430 core
/*
* Copyright 2018 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

out vec4 color;

uniform vec3 camera_world;
uniform float brightness;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 

uniform float specularAlpha;

uniform sampler2D diffuseTexture;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

uniform struct PointLight1 {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL1;

uniform struct PointLight2 {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL2;

uniform struct PointLight3 {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL3;

uniform struct PointLight4 {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL4;

uniform struct PointLight5 {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL5;

uniform struct PointLight6 {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL6;

uniform struct PointLight7 {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL7;
uniform struct PointLight8 {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL8;
uniform struct PointLight9 {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL9;
uniform struct PointLight10 {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL10;
uniform struct PointLight11 {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL11;
uniform struct PointLight12 {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL12;


vec3 cel(vec3 color, vec3 normal, vec3 lightDir){

float intensity;

intensity = dot(lightDir,normal);

vec3 outCol;

	if (intensity > 0.95)
		//outCol = vec3(0.7,0.7,0.7);
		outCol = color *0.9;
    else if (intensity > 0.75)
	    //outCol = vec3(0.5,0.5,0.5);
	    outCol = color *0.7;
	else if (intensity > 0.5)
        outCol = color *0.5;
		//outCol = vec3(0.3,0.3,0.3);
	else if (intensity > 0.25)
		outCol = color *0.2;
		//outCol = vec3(0.2,0.2,0.2);
	else
		outCol = color *0.005;
		//outCol = vec3(0.1,0.1,0.1);
	
	return outCol;
}


vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	return (diffuseF * diffuseC * max(0, dot(n, l)) + specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att; 
}

void main() {	
	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);
	vec3 lightDir = normalize(-dirL.direction);
	
	vec3 texColor = texture(diffuseTexture, vert.uv).rgb;
	//color = vec4(texColor * materialCoefficients.x, 1); // ambient

	color = vec4(texColor, 1);

	color.rgb = cel(color.rgb*dirL.color, n, lightDir);
	color.rgb *= vec3(brightness, brightness, brightness);

	// add directional light contribution
	color.rgb += phong(n, -dirL.direction, v, dirL.color * texColor, materialCoefficients.y, dirL.color, materialCoefficients.z, specularAlpha, false, vec3(0));
	
	// add point light1 contribution
	color.rgb += phong(n, pointL1.position - vert.position_world, v, pointL1.color * texColor, materialCoefficients.y, pointL1.color, materialCoefficients.z, specularAlpha, true, pointL1.attenuation);

	// add point light2 contribution
	color.rgb += phong(n, pointL2.position - vert.position_world, v, pointL2.color * texColor, materialCoefficients.y, pointL2.color, materialCoefficients.z, specularAlpha, true, pointL2.attenuation);

	// add point light3 contribution
	color.rgb += phong(n, pointL3.position - vert.position_world, v, pointL3.color * texColor, materialCoefficients.y, pointL3.color, materialCoefficients.z, specularAlpha, true, pointL3.attenuation);

	// add point light4 contribution
	color.rgb += phong(n, pointL4.position - vert.position_world, v, pointL4.color * texColor, materialCoefficients.y, pointL4.color, materialCoefficients.z, specularAlpha, true, pointL4.attenuation);

	// add point light5 contribution
	color.rgb += phong(n, pointL5.position - vert.position_world, v, pointL5.color * texColor, materialCoefficients.y, pointL5.color, materialCoefficients.z, specularAlpha, true, pointL5.attenuation);

	// add point light6 contribution
	color.rgb += phong(n, pointL6.position - vert.position_world, v, pointL6.color * texColor, materialCoefficients.y, pointL6.color, materialCoefficients.z, specularAlpha, true, pointL6.attenuation);

	// add point light7 contribution
	color.rgb += phong(n, pointL7.position - vert.position_world, v, pointL7.color * texColor, materialCoefficients.y, pointL7.color, materialCoefficients.z, specularAlpha, true, pointL7.attenuation);

	// add point light8 contribution
	color.rgb += phong(n, pointL8.position - vert.position_world, v, pointL8.color * texColor, materialCoefficients.y, pointL8.color, materialCoefficients.z, specularAlpha, true, pointL8.attenuation);

	// add point light9 contribution
	color.rgb += phong(n, pointL9.position - vert.position_world, v, pointL9.color * texColor, materialCoefficients.y, pointL9.color, materialCoefficients.z, specularAlpha, true, pointL9.attenuation);

	// add point light10 contribution
	color.rgb += phong(n, pointL10.position - vert.position_world, v, pointL10.color * texColor, materialCoefficients.y, pointL10.color, materialCoefficients.z, specularAlpha, true, pointL10.attenuation);

	// add point light11 contribution
	color.rgb += phong(n, pointL11.position - vert.position_world, v, pointL11.color * texColor, materialCoefficients.y, pointL11.color, materialCoefficients.z, specularAlpha, true, pointL11.attenuation);

	// add point light12 contribution
	color.rgb += phong(n, pointL12.position - vert.position_world, v, pointL12.color * texColor, materialCoefficients.y, pointL12.color, materialCoefficients.z, specularAlpha, true, pointL12.attenuation);

	color.rgb=color.rgb*brightness;

}
